//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2009 Hartmut Kaiser
//  Copyright (c) 2005 Michel Zandstra (michelzandstra@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_IMPL_ENGINE_SESSION_HPP
#define SAGA_IMPL_ENGINE_SESSION_HPP

#include <list>

#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

#include <saga/impl/config.hpp>
#include <saga/impl/object.hpp>
#include <saga/impl/engine/engine.hpp>

#include <saga/saga/context.hpp>
#include <saga/saga/uuid.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace impl {

    // forward declaration only
    struct base_context_list;

#if defined(__AIX__) && defined(__GNUC__)     // see comment in saga/impl_base.hpp
    struct aix_hack
    {
        aix_hack(bool is_default)
          : global_di_called_(is_default)
        {
            if (is_default)
                _GLOBAL__DI(); 
        }
        ~aix_hack()
        {
            if (global_di_called_)
                _GLOBAL__DD(); 
        }

        bool global_di_called_;
    };
#else
    struct aix_hack
    {
        aix_hack(bool is_default) {}
        ~aix_hack() {}
    };
#endif

    /**
    * Session: provides the functionality of a session handle.
    *
    * Multiple context's can be added to the session handle. Session 
    * also handles the shared adaptor data of the session, and 
    * contains an instance of the saga::engine that is used by 
    * the proxy classes.
    *
    * @see Saga API, adaptor_data/instance_data, engine
    */
    class session : public saga::impl::object
    {
    private:
        typedef v1_0::preference_type preference_type;
        typedef v1_0::cpi cpi;

        /*! the list of contexts */
        mutable base_context_list* contexts_;
        bool is_default_;

        aix_hack global_di_aix_;

        /*! the instance of the saga::engine */
        engine const engine_;

        session* This() { return this; }

    public:
        /**
        * Constructor / Destructor
        */
        session (bool is_default) 
          : object(saga::object::Session), 
            contexts_(NULL), is_default_(is_default),
            global_di_aix_(is_default),
            engine_(This())
        {}
        ~session (void);

        // saga::object interface
        saga::object clone() const; 

        // test if this is the a default session
        SAGA_EXPORT bool is_default_session() const
        {
            return is_default_;
        }

        SAGA_EXPORT void add_proto_context (
            std::vector<std::pair<std::string, std::string> > const& entries);

        SAGA_EXPORT void add_proto_context (
            std::vector<std::pair<std::string, std::string> > const& entries,
            std::vector<std::pair<std::string, std::vector<std::string> > > const& vector_entries);

        /**
        * Returns the context attached to the session
        *
        * @return  the context list
        */
        std::vector<saga::context> list_contexts() const;

        /**
        * Removes a context from the context list
        *
        * @param the context to be added
        */
        SAGA_EXPORT void remove_context (saga::context const& c);

        /**
        * Adds the context to the context list
        *
        * @param the context to be added
        */
        SAGA_EXPORT void add_context (saga::context const& c);

        // release all contexts stored in the context list
        // this is a helper function called during shutdown to break the 
        // circular dependency of the session and the held contexts
        void release_contexts();

        /**
        * Creates an adaptor object implementing the interface
        *
        * @param cpi_name   a string specifying the interface for which an object 
        *                   is needed
        * @param op_name    a description method wanted from the adaptor
        * @param prefs      a description table describing the adaptor needed
        *
        * @return a pointer of type T* to the created object, or NULL of no suitable 
        *         adaptor could be found
        *
        * @see saga::adaptor_selector::select_adaptor
        */
        v1_0::cpi* get_adaptor (std::string     const & cpi_name,
                                std::string     const & op_name, 
                                preference_type const & prefs,
                                proxy                 * proxy_, 
                                adaptor_selector::adaptor_info_list_type & no_no_list,
                                v1_0::op_info         & oi) const
        {
            return engine_.get_adaptor (cpi_name, op_name, prefs, no_no_list, proxy_, oi);
        }

        /**
        * Tests, whether a given adaptor object implements the required interface
        *
        * @param adaptor    a pointer to the adaptor to test
        * @param cpi_name   a string specifying the interface for which an object 
        *                   is needed
        * @param op_name    a description method wanted from the adaptor
        * @param prefs      a description table describing the adaptor needed
        *
        * @return a pointer of type T* to the created object, or NULL of no suitable 
        *         adaptor could be found
        *
        * @see saga::adaptor_selector::select_adaptor
        */
        bool test_adaptor (v1_0::cpi const* adaptor, 
            std::string const& cpi_name, std::string const& op_name,
            preference_type const& prefs, v1_0::op_info& oi) const
        {
            return engine_.test_adaptor(adaptor, cpi_name, op_name, prefs, oi);
        }

        // return the name of the adaptor with the given adaptor id
        std::string get_adaptor_name(saga::uuid id) const
        {
            return engine_.get_adaptor_name(id);
        }

    }; // class session

///////////////////////////////////////////////////////////////////////////////
}}  // namespace saga::impl

#endif // SAGA_IMPL_ENGINE_SESSION_HPP

