//  Copyright (c) 2005-2009 Hartmut Kaiser
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_IMPL_EXCEPTION_HPP
#define SAGA_IMPL_EXCEPTION_HPP

// filesystem will be in TR2
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/exception.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/convenience.hpp>

#include <boost/preprocessor/stringize.hpp>

#include <saga/saga/base.hpp>
#include <saga/saga/object.hpp>
#include <saga/saga/util.hpp>
#include <saga/saga/error.hpp>
#include <saga/saga/path_leaf.hpp>
#include <saga/impl/exception_list.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
    ///////////////////////////////////////////////////////////////////////////
    namespace impl 
    {
        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::exception 
        SAGA_EXPORT void throw_exception(saga::object const& obj, 
            std::string const& m, error e);

        SAGA_EXPORT void throw_exception(saga::object const& obj, 
            exception_list const& l);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::exception from a saga object member function
        SAGA_EXPORT void throw_exception(saga::impl::object const *this_, 
            std::string const& m, error e);

        SAGA_EXPORT void throw_exception(saga::impl::object const *this_, 
            exception_list const& l);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::exception from an adaptor member function
        SAGA_EXPORT void throw_exception(saga::impl::v1_0::cpi const *this_, 
            std::string const& m, error e);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::exception from any other function
        SAGA_EXPORT void throw_exception(void const *this_, 
            std::string const& m, error e);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::adaptors::exception 
        SAGA_EXPORT void throw_exception(saga::object const& obj, 
            std::string const& m, adaptors::error e);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::exception from a saga object member function
        SAGA_EXPORT void throw_exception(saga::impl::object const *this_, 
            std::string const& m, adaptors::error e);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::exception from an adaptor member function
        SAGA_EXPORT void throw_exception(saga::impl::v1_0::cpi const *this_, 
            std::string const& m, adaptors::error e);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::exception from any other function
        SAGA_EXPORT void throw_exception(void const *this_, 
            std::string const& m, adaptors::error e);

    ///////////////////////////////////////////////////////////////////////////
    }

    ///////////////////////////////////////////////////////////////////////////
    namespace adaptors 
    {
        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::adaptors::exception
        SAGA_EXPORT void throw_adaptor_exception(
            saga::impl::v1_0::cpi const *this_, 
            std::string const& adaptor_name, std::string const& m, 
            saga::error e);

        SAGA_EXPORT void throw_adaptor_exception(
            saga::impl::v1_0::cpi const *this_, impl::exception_list const& l);

        SAGA_EXPORT void throw_adaptor_exception(
            std::string const& adaptor_name, std::string const& m, 
            saga::error e);

        ///////////////////////////////////////////////////////////////////////
        //  throw a saga::adaptors::exception
        SAGA_EXPORT void throw_adaptor_exception(
            saga::impl::v1_0::cpi const *this_, 
            std::string const& adaptor_name, std::string const& m, 
            saga::adaptors::error e);

        SAGA_EXPORT void throw_adaptor_exception(
            std::string const& adaptor_name, std::string const& m, 
            saga::adaptors::error e);

    ///////////////////////////////////////////////////////////////////////////
    }

///////////////////////////////////////////////////////////////////////////////
}   // namespace saga

///////////////////////////////////////////////////////////////////////////////
#define SAGA_THROW_PLAIN(obj, msg, errcode)                                   \
        saga::impl::throw_exception(obj, msg, errcode);                       \
    /**/

#define SAGA_THROW_PLAIN_LIST(obj, list)                                      \
        saga::impl::throw_exception(obj, list);                               \
    /**/

///////////////////////////////////////////////////////////////////////////////
#define SAGA_THROW_VERBATIM(obj, msg, errcode)                                \
    {                                                                         \
        std::string __s("");                                                  \
        SAGA_VERBOSE (SAGA_VERBOSE_LEVEL_DEBUG)                               \
        {                                                                     \
            boost::filesystem::path __p(__FILE__, boost::filesystem::native); \
            __s = saga::detail::leaf(__p);                                    \
            __s += std::string("(") + BOOST_PP_STRINGIZE(__LINE__) + "): ";   \
        }                                                                     \
        __s += std::string("") + msg;                                         \
        SAGA_THROW_PLAIN(obj, __s, errcode);                                  \
    }                                                                         \
    /**/
    
///////////////////////////////////////////////////////////////////////////////
#define SAGA_THROW(msg, errcode)                                              \
    SAGA_THROW_VERBATIM(this, msg, errcode)                                   \
    /**/
    
///////////////////////////////////////////////////////////////////////////////
#define SAGA_THROW_NO_OBJECT(msg, errcode)                                    \
    SAGA_THROW_VERBATIM(saga::object(), msg, errcode)                         \
    /**/

///////////////////////////////////////////////////////////////////////////////
#define SAGA_ADAPTOR_THROW_PLAIN(obj, msg, errcode)                           \
        saga::adaptors::throw_adaptor_exception(obj,                          \
            BOOST_PP_STRINGIZE(SAGA_ADAPTOR_NAME), msg, errcode);             \
    /**/

#define SAGA_ADAPTOR_THROW_PLAIN_LIST(obj, list)                              \
        saga::adaptors::throw_adaptor_exception(obj, list);                   \
    /**/

///////////////////////////////////////////////////////////////////////////////
#define SAGA_ADAPTOR_THROW_VERBATIM(cpi, msg, errcode)                        \
    {                                                                         \
        std::string __s("");                                                  \
        SAGA_VERBOSE (SAGA_VERBOSE_LEVEL_DEBUG)                               \
        {                                                                     \
            boost::filesystem::path __p(__FILE__, boost::filesystem::native); \
            __s = saga::detail::leaf(__p);                                    \
            __s += std::string("(") + BOOST_PP_STRINGIZE(__LINE__) + "): ";   \
        }                                                                     \
        __s += std::string("") + msg;                                         \
        SAGA_ADAPTOR_THROW_PLAIN(cpi, __s, errcode);                          \
    }                                                                         \
    /**/

///////////////////////////////////////////////////////////////////////////////
#define SAGA_ADAPTOR_THROW(msg, errcode)                                      \
    SAGA_ADAPTOR_THROW_VERBATIM(this, msg, errcode)                           \
    /**/
    
///////////////////////////////////////////////////////////////////////////////
#define SAGA_ADAPTOR_THROW_NO_CONTEXT(msg, errcode)                           \
    {                                                                         \
        std::string __s("");                                                  \
        SAGA_VERBOSE (SAGA_VERBOSE_LEVEL_DEBUG)                               \
        {                                                                     \
            boost::filesystem::path __p(__FILE__, boost::filesystem::native); \
            __s = saga::detail::leaf(__p);                                    \
            __s += std::string("(") + BOOST_PP_STRINGIZE(__LINE__) + "): ";   \
        }                                                                     \
        __s += std::string("") + msg;                                         \
        saga::adaptors::throw_adaptor_exception(                              \
              BOOST_PP_STRINGIZE(SAGA_ADAPTOR_NAME), __s, errcode);           \
    }                                                                         \
    /**/

#endif // !SAGA_IMPL_EXCEPTION_HPP

