//  Copyright (c) 2005-2009 Hartmut Kaiser
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if !defined(SAGA_IMPL_JOB_SPLIT_COMMANDLINE_HK080305_0940AM)
#define SAGA_IMPL_JOB_SPLIT_COMMANDLINE_HK080305_0940AM

#include <vector>
#include <string>

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace impl 
{
    // The command line splitting below is taken from the Boost.ProgramOptions
    // library.
    
    // Copyright Vladimir Prus 2002-2004.
    // Distributed under the Boost Software License, Version 1.0.
    // (See accompanying file LICENSE_1_0.txt
    // or copy at http://www.boost.org/LICENSE_1_0.txt)

    // Take a command line string and splits in into tokens, according
    // to the following rules
    //
    //  - Arguments are delimited by white space, which is either a space or 
    //    a tab.
    //  - A string surrounded by double quotation marks is interpreted as a 
    //    single argument, regardless of white space contained within. A 
    //    quoted string can be embedded in an argument. 
    //  - A double quotation mark preceded by a backslash, \", is interpreted 
    //    as a literal double quotation mark (").
    //  - Backslashes are interpreted literally, unless they immediately precede 
    //    a double quotation mark.
    //
    // These rules are a slight modification of the windows command line 
    // processor rules, as described here:
    //    http://article.gmane.org/gmane.comp.lib.boost.user/3005
    //    http://msdn.microsoft.com/library/en-us/vccelng/htm/progs_12.asp
    inline std::vector<std::string> 
    split_commandline (std::string const& input)
    {
        using namespace std;
        vector <string> result;

        string::const_iterator i = input.begin ();
        string::const_iterator e = input.end   ();

        for ( /**/ ; i != e; ++i )
            if ( ! isspace ((unsigned char)*i) )
                break;
       
        if ( i != e ) 
        {
          string current;
          bool   inside_quoted   = false;
          int    backslash_count = 0;

          for (/**/; i != e; ++i) 
          {
            if (*i == '\\') 
            {
              // just count backslashes 
              ++backslash_count;
            }
            else if ( *i == '"' ) 
            {
              // '"' preceded by a backslash is a literal quote
              // the backslash which quoted is removed
              if ( backslash_count > 0 ) 
              {
                current += '"';
                --backslash_count;
              }
              // '"' not preceded by a backslash limits a quote
              else 
              {
                inside_quoted = ! inside_quoted;
              }
            } 
            else 
            {
              // Not quote or backslash. All accumulated backslashes should be
              // added
              if ( backslash_count ) 
              {
                current.append (backslash_count, '\\');
                backslash_count = 0;
              }

              if ( isspace ((unsigned char) *i) && ! inside_quoted ) 
              {
                // Space outside quoted section terminate the current argument
                result.push_back (current);
                current.resize   (0);
                
                for ( /**/ ; i != e && isspace ((unsigned char) *i); ++i ) 
                    /**/;

                --i;
              } 
              else 
              {                  
                current += *i;
              }
            }
          }

          // If we have trailing backslashes, add them
          if ( backslash_count )
          {
            current.append (backslash_count, '\\');
          }

          // If we have non-empty 'current' or we're still in quoted
          // section (even if 'current' is empty), add the last token.
          if ( ! current.empty () || inside_quoted )
          {
            result.push_back (current);        
          }
          // FIXME: we SHOULD thrown an exception here, non-matching quotes
          // are a BadParameter -- AM
        }
        return result;
    }

///////////////////////////////////////////////////////////////////////////////
}}

#endif 

